@@ -8,14 +8,15 @@ import logging
 import os
 import shutil
 import socket
-import textwrap
 import time
 from concurrent.futures import ThreadPoolExecutor
 from contextlib import suppress
+from pathlib import Path
 from typing import Any, Callable, Coroutine, Dict, List, Optional, Set, Tuple, Union
 
 import adaptive
-import dill
+import cloudpickle
+import jinja2
 import pandas as pd
 import structlog
 import zmq
@@ -24,8 +25,15 @@ import zmq.ssh
 from tinydb import Query, TinyDB
 
 from adaptive_scheduler.scheduler import BaseScheduler
-from adaptive_scheduler.utils import _progress, _remove_or_move_files, load_parallel
-from adaptive_scheduler.widgets import log_explorer
+from adaptive_scheduler.utils import (
+    _deserialize,
+    _progress,
+    _remove_or_move_files,
+    _serialize,
+    load_parallel,
+    maybe_lst,
+)
+from adaptive_scheduler.widgets import info
 
 ctx = zmq.asyncio.Context()
 
@@ -84,6 +92,8 @@ class DatabaseManager(_BaseManager):
         A scheduler instance from `adaptive_scheduler.scheduler`.
     db_fname : str
         Filename of the database, e.g. 'running.json'.
+    learners : list of `adaptive.BaseLearner` isinstances
+        List of `learners` corresponding to `fnames`.
     fnames : list
         List of `fnames` corresponding to `learners`.
     overwrite_db : bool, default: True
@@ -100,13 +110,15 @@ class DatabaseManager(_BaseManager):
         url: str,
         scheduler: BaseScheduler,
         db_fname: str,
-        fnames: List[str],
+        learners: List[adaptive.BaseLearner],
+        fnames: Union[List[str], List[List[str]]],
         overwrite_db: bool = True,
-    ):
+    ) -> None:
         super().__init__()
         self.url = url
         self.scheduler = scheduler
         self.db_fname = db_fname
+        self.learners = learners
         self.fnames = fnames
         self.overwrite_db = overwrite_db
 
@@ -141,7 +153,7 @@ class DatabaseManager(_BaseManager):
     def n_done(self) -> int:
         Entry = Query()
         with TinyDB(self.db_fname) as db:
-            return db.count(Entry.is_done == True)  # noqa: E711
+            return db.count(Entry.is_done == True)  # noqa: E712
 
     def create_empty_db(self) -> None:
         """Create an empty database that keeps track of
@@ -179,8 +191,8 @@ class DatabaseManager(_BaseManager):
                     "warning in the [mpi4py](https://bit.ly/2HAk0GG) documentation."
                 )
             entry = db.get(
-                (Entry.job_id == None) & (Entry.is_done == False)
-            )  # noqa: E711
+                (Entry.job_id == None) & (Entry.is_done == False)  # noqa: E711
+            )
             log.debug("choose fname", entry=entry)
             if entry is None:
                 return None
@@ -195,12 +207,24 @@ class DatabaseManager(_BaseManager):
             )
         return entry["fname"]
 
-    def _stop_request(self, fname: str) -> None:
+    def _stop_request(self, fname: Union[str, List[str]]) -> None:
+        fname = maybe_lst(fname)  # if a BalancingLearner
         Entry = Query()
         with TinyDB(self.db_fname) as db:
             reset = dict(job_id=None, is_done=True, job_name=None)
+            assert (
+                db.get(Entry.fname == fname) is not None
+            )  # make sure the entry exists
             db.update(reset, Entry.fname == fname)
 
+    def _stop_requests(self, fnames: List[Union[str, List[str]]]) -> None:
+        # Same as `_stop_request` but optimized for processing many `fnames` at once
+        fnames = {str(maybe_lst(fname)) for fname in fnames}
+        with TinyDB(self.db_fname) as db:
+            reset = dict(job_id=None, is_done=True, job_name=None)
+            doc_ids = [e.doc_id for e in db.all() if str(e["fname"]) in fnames]
+            db.update(reset, doc_ids=doc_ids)
+
     def _dispatch(self, request: Tuple[str, ...]) -> Union[str, Exception, None]:
         request_type, *request_arg = request
         log.debug("got a request", request=request)
@@ -211,8 +235,15 @@ class DatabaseManager(_BaseManager):
                 kwargs = dict(job_id=job_id, log_fname=log_fname, job_name=job_name)
                 # give the worker a job and send back the fname to the worker
                 fname = self._start_request(**kwargs)
+                if fname is None:
+                    raise RuntimeError("No more learners to run in the database.")
+                learner = next(
+                    learner
+                    for learner, f in zip(self.learners, self.fnames)
+                    if maybe_lst(f) == fname
+                )
                 log.debug("choose a fname", fname=fname, **kwargs)
-                return fname
+                return learner, fname
             elif request_type == "stop":
                 fname = request_arg[0]  # workers send us the fname they were given
                 log.debug("got a stop request", fname=fname)
@@ -233,9 +264,9 @@ class DatabaseManager(_BaseManager):
         socket.bind(self.url)
         try:
             while True:
-                self._last_request = await socket.recv_pyobj()
+                self._last_request = await socket.recv_serialized(_deserialize)
                 self._last_reply = self._dispatch(self._last_request)
-                await socket.send_pyobj(self._last_reply)
+                await socket.send_serialized(self._last_reply, _serialize)
         finally:
             socket.close()
 
@@ -495,7 +526,6 @@ def get_allowed_url() -> str:
 
 def _make_default_run_script(
     url: str,
-    learners_file: str,
     save_interval: int,
     log_interval: int,
     goal: Union[Callable[[adaptive.BaseLearner], bool], None] = None,
@@ -505,110 +535,50 @@ def _make_default_run_script(
 ) -> None:
     default_runner_kwargs = dict(shutdown_executor=True)
     runner_kwargs = dict(default_runner_kwargs, goal=goal, **(runner_kwargs or {}))
-    serialized_runner_kwargs = dill.dumps(runner_kwargs)
+    serialized_runner_kwargs = cloudpickle.dumps(runner_kwargs)
 
-    if executor_type == "mpi4py":
-        import_line = "from mpi4py.futures import MPIPoolExecutor"
-        executor_line = "MPIPoolExecutor()"
-    elif executor_type == "ipyparallel":
-        import_line = "from adaptive_scheduler.utils import connect_to_ipyparallel"
-        executor_line = "connect_to_ipyparallel(profile=args.profile, n=args.n)"
-    elif executor_type == "dask-mpi":
+    if executor_type not in ("mpi4py", "ipyparallel", "dask-mpi", "process-pool"):
+        raise NotImplementedError(
+            "Use 'ipyparallel', 'dask-mpi', 'mpi4py' or 'process-pool'."
+        )
+
+    if executor_type == "dask-mpi":
         try:
             import dask_mpi  # noqa: F401
         except ModuleNotFoundError as e:
             msg = "You need to have 'dask-mpi' installed to use `executor_type='dask-mpi'`."
             raise Exception(msg) from e
-        import_line = "from distributed import Client"
-        executor_line = "Client()"
-    elif executor_type == "process-pool":
-        import_line = "from concurrent.futures import ProcessPoolExecutor"
-        executor_line = "ProcessPoolExecutor(max_workers=args.n)"
-    else:
-        raise NotImplementedError(
-            "Use 'ipyparallel', 'dask-mpi', 'mpi4py' or 'process-pool'."
-        )
 
-    if os.path.abspath(os.path.dirname(learners_file)) != os.path.abspath(""):
-        raise RuntimeError(
-            f"The {learners_file} needs to be in the same"
-            " directory as where this is run from."
-        )
+    with open(Path(__file__).parent / "run_script.py.j2") as f:
+        empty = "".join(f.readlines())
 
-    learners_module = os.path.splitext(os.path.basename(learners_file))[0]
-
-    template = textwrap.dedent(
-        f"""\
-    #!/usr/bin/env python3
-    # {run_script_fname}, automatically generated
-    # by `adaptive_scheduler.server_support._make_default_run_script()`.
-    import argparse
-    from contextlib import suppress
-
-    import adaptive
-    import dill
-    from adaptive_scheduler import client_support
-    {import_line}
-
-
-    # the file that defines the learners we created above
-    from {learners_module} import learners, fnames
-
-    if __name__ == "__main__":  # ← use this, see warning @ https://bit.ly/2HAk0GG
-
-        # parse arguments
-        parser = argparse.ArgumentParser()
-        parser.add_argument("--profile", action="store", dest="profile", type=str)
-        parser.add_argument("--n", action="store", dest="n", type=int)
-        parser.add_argument("--log-fname", action="store", dest="log_fname", type=str)
-        parser.add_argument("--job-id", action="store", dest="job_id", type=str)
-        parser.add_argument("--name", action="store", dest="name", type=str)
-        args = parser.parse_args()
-
-        # the address of the "database manager"
-        url = "{url}"
-
-        # ask the database for a learner that we can run which we log in `args.log_fname`
-        learner, fname = client_support.get_learner(
-            learners, fnames, url, args.log_fname, args.job_id, args.name
-        )
-
-        # load the data
-        with suppress(Exception):
-            learner.load(fname)
-
-        # connect to the executor
-        executor = {executor_line}
-
-        # this is serialized by dill.dumps
-        runner_kwargs = dill.loads({serialized_runner_kwargs})
-
-        # run until `some_goal` is reached with an `MPIPoolExecutor`
-        runner = adaptive.Runner(learner, executor=executor, **runner_kwargs)
-
-        # periodically save the data (in case the job dies)
-        runner.start_periodic_saving(dict(fname=fname), interval={save_interval})
-
-        # log progress info in the job output script, optional
-        client_support.log_info(runner, interval={log_interval})
-
-        # block until runner goal reached
-        runner.ioloop.run_until_complete(runner.task)
-
-        # save once more after the runner is done
-        learner.save(fname)
-
-        # tell the database that this learner has reached its goal
-        client_support.tell_done(url, fname)
-    """
+    template = jinja2.Template(empty).render(
+        run_script_fname=run_script_fname,
+        executor_type=executor_type,
+        url=url,
+        serialized_runner_kwargs=serialized_runner_kwargs,
+        save_interval=save_interval,
+        log_interval=log_interval,
     )
-    if executor_type == "dask-mpi":
-        template = "from dask_mpi import initialize; initialize()\n" + template
 
     with open(run_script_fname, "w") as f:
         f.write(template)
 
 
+def _get_infos(fname: str, only_last: bool = True) -> List[str]:
+    status_lines: List[str] = []
+    with open(fname) as f:
+        lines = f.readlines()
+        for line in reversed(lines):
+            with suppress(Exception):
+                info = json.loads(line)
+                if info["event"] == "current status":
+                    status_lines.append(info)
+                    if only_last:
+                        return status_lines
+        return status_lines
+
+
 def parse_log_files(
     job_names: List[str],
     database_manager: DatabaseManager,
@@ -639,39 +609,26 @@ def parse_log_files(
     _queue = scheduler.queue()
     database_manager.update(_queue)
 
-    def _get_infos(fname: str, only_last: bool = True):
-        status_lines: List[str] = []
-        with open(fname) as f:
-            lines = f.readlines()
-            for line in reversed(lines):
-                with suppress(Exception):
-                    info = json.loads(line)
-                    if info["event"] == "current status":
-                        status_lines.append(info)
-                        if only_last:
-                            return status_lines
-            return status_lines
-
     infos = []
     for entry in database_manager.as_dicts():
         log_fname = entry["log_fname"]
-        if log_fname is None:
+        if log_fname is None or not os.path.exists(log_fname):
             continue
-        for info in _get_infos(log_fname, only_last):
-            info.pop("event")  # this is always "current status"
-            info["timestamp"] = datetime.datetime.strptime(
-                info["timestamp"], "%Y-%m-%d %H:%M.%S"
+        for info_dict in _get_infos(log_fname, only_last):
+            info_dict.pop("event")  # this is always "current status"
+            info_dict["timestamp"] = datetime.datetime.strptime(
+                info_dict["timestamp"], "%Y-%m-%d %H:%M.%S"
             )
-            info["elapsed_time"] = pd.to_timedelta(info["elapsed_time"])
-            info.update(entry)
-            infos.append(info)
+            info_dict["elapsed_time"] = pd.to_timedelta(info_dict["elapsed_time"])
+            info_dict.update(entry)
+            infos.append(info_dict)
 
-    for info in infos:
-        info_from_queue = _queue.get(info["job_id"])
+    for info_dict in infos:
+        info_from_queue = _queue.get(info_dict["job_id"])
         if info_from_queue is None:
             continue
-        info["state"] = info_from_queue["state"]
-        info["job_name"] = info_from_queue["job_name"]
+        info_dict["state"] = info_from_queue["state"]
+        info_dict["job_name"] = info_from_queue["job_name"]
 
     return pd.DataFrame(infos)
 
@@ -683,7 +640,14 @@ def _get_all_files(job_names: List[str], scheduler: BaseScheduler) -> List[str]:
     batch_fnames = [scheduler.batch_fname(name) for name in job_names]
     fnames = log_fnames + output_fnames + batch_fnames
     all_files = [glob.glob(f.replace(scheduler._JOB_ID_VARIABLE, "*")) for f in fnames]
-    return sum(all_files, [])
+    all_files = sum(all_files, [])
+
+    # For schedulers that use a single batch file
+    name_prefix = job_names[0].rsplit("-", 1)[0]
+    batch_file = scheduler.batch_fname(name_prefix)
+    if os.path.exists(batch_file):
+        all_files.append(batch_file)
+    return all_files
 
 
 def cleanup(
@@ -751,17 +715,21 @@ class RunManager(_BaseManager):
     ----------
     scheduler : `~adaptive_scheduler.scheduler.BaseScheduler`
         A scheduler instance from `adaptive_scheduler.scheduler`.
+    learners : list of `adaptive.BaseLearner` isinstances
+        List of `learners` corresponding to `fnames`.
+    fnames : list
+        List of `fnames` corresponding to `learners`.
     goal : callable, default: None
         The goal passed to the `adaptive.Runner`. Note that this function will
         be serialized and pasted in the ``run_script``.
+    check_goal_on_start : bool, default: True
+        Checks whether a learner is already done. Only works if the learner is loaded.
     runner_kwargs : dict, default: None
         Extra keyword argument to pass to the `adaptive.Runner`. Note that this dict
         will be serialized and pasted in the ``run_script``.
     url : str, default: None
         The url of the database manager, with the format
         ``tcp://ip_of_this_machine:allowed_port.``. If None, a correct url will be chosen.
-    learners_file : str, default: "learners_file.py"
-        The module that defined the variables ``learners`` and ``fnames``.
     save_interval : int, default: 300
         Time in seconds between saving of the learners.
     log_interval : int, default: 300
@@ -792,8 +760,6 @@ class RunManager(_BaseManager):
 
     Attributes
     ----------
-    learners_module : module
-        Attribute access to the ``learners_file``.
     job_names : list
         List of job_names. Generated with ``self.job_name``.
     database_manager : `DatabaseManager`
@@ -828,7 +794,6 @@ class RunManager(_BaseManager):
     ...     return learner.npoints > 2000
     >>> run_manager = adaptive_scheduler.server_support.RunManager(
     ...     scheduler=scheduler,
-    ...     learners_file="learners_file.py",
     ...     goal=goal,
     ...     log_interval=30,
     ...     save_interval=30,
@@ -840,10 +805,12 @@ class RunManager(_BaseManager):
     def __init__(
         self,
         scheduler: BaseScheduler,
+        learners: List[adaptive.BaseLearner],
+        fnames: List[str],
         goal: Union[Callable[[adaptive.BaseLearner], bool], None] = None,
+        check_goal_on_start: bool = True,
         runner_kwargs: Optional[dict] = None,
         url: Optional[str] = None,
-        learners_file: str = "learners_file.py",
         save_interval: int = 300,
         log_interval: int = 300,
         job_name: str = "adaptive-scheduler",
@@ -851,7 +818,7 @@ class RunManager(_BaseManager):
         kill_interval: int = 60,
         kill_on_error: Union[str, Callable[[List[str]], bool], None] = "srun: error:",
         move_old_logs_to: Optional[str] = "old_logs",
-        db_fname: str = "running.json",
+        db_fname: Optional[str] = None,
         overwrite_db: bool = True,
         job_manager_kwargs: Optional[Dict[str, Any]] = None,
         kill_manager_kwargs: Optional[Dict[str, Any]] = None,
@@ -860,8 +827,8 @@ class RunManager(_BaseManager):
         # Set from arguments
         self.scheduler = scheduler
         self.goal = goal
+        self.check_goal_on_start = check_goal_on_start
         self.runner_kwargs = runner_kwargs
-        self.learners_file = learners_file
         self.save_interval = save_interval
         self.log_interval = log_interval
         self.job_name = job_name
@@ -869,7 +836,7 @@ class RunManager(_BaseManager):
         self.kill_interval = kill_interval
         self.kill_on_error = kill_on_error
         self.move_old_logs_to = move_old_logs_to
-        self.db_fname = db_fname
+        self.db_fname = db_fname or f"{job_name}-database.json"
         self.overwrite_db = overwrite_db
         self.job_manager_kwargs = job_manager_kwargs or {}
         self.kill_manager_kwargs = kill_manager_kwargs or {}
@@ -879,16 +846,23 @@ class RunManager(_BaseManager):
         self.end_time: Optional[float] = None
 
         # Set on init
-        self.learners_module = self._get_learners_file()
-        self.job_names = [
-            f"{self.job_name}-{i}" for i in range(len(self.learners_module.learners))
-        ]
+        self.learners = learners
+        self.fnames = fnames
+
+        if isinstance(self.fnames[0], (list, tuple)):
+            # For a BalancingLearner
+            assert isinstance(self.fnames[0][0], str)
+        else:
+            assert isinstance(self.fnames[0], str)
+
+        self.job_names = [f"{self.job_name}-{i}" for i in range(len(self.learners))]
         self.url = url or get_allowed_url()
         self.database_manager = DatabaseManager(
             self.url,
             self.scheduler,
             self.db_fname,
-            self.learners_module.fnames,
+            self.learners,
+            self.fnames,
             self.overwrite_db,
         )
         self.job_manager = JobManager(
@@ -914,7 +888,6 @@ class RunManager(_BaseManager):
     def _setup(self) -> None:
         _make_default_run_script(
             self.url,
-            self.learners_file,
             self.save_interval,
             self.log_interval,
             self.goal,
@@ -923,6 +896,15 @@ class RunManager(_BaseManager):
             self.scheduler.executor_type,
         )
         self.database_manager.start()
+        if self.check_goal_on_start:
+            # Check if goal already reached
+            # Only works after the `database_manager` has started.
+            done_fnames = [
+                fname
+                for fname, learner in zip(self.fnames, self.learners)
+                if self.goal(learner)
+            ]
+            self.database_manager._stop_requests(done_fnames)
         self.job_manager.start()
         if self.kill_manager:
             self.kill_manager.start()
@@ -932,24 +914,17 @@ class RunManager(_BaseManager):
         await self.job_manager.task
         self.end_time = time.time()
 
-    def _get_learners_file(self):
-        from importlib.util import module_from_spec, spec_from_file_location
-
-        spec = spec_from_file_location("learners_file", self.learners_file)
-        learners_file = module_from_spec(spec)
-        spec.loader.exec_module(learners_file)
-        return learners_file
-
     def cancel(self) -> None:
         """Cancel the manager tasks and the jobs in the queue."""
         self.database_manager.cancel()
         self.job_manager.cancel()
         self.kill_manager.cancel()
         self.scheduler.cancel(self.job_names)
-        self.task.cancel()
+        if self.task is not None:
+            self.task.cancel()
         self.end_time = time.time()
 
-    def cleanup(self) -> None:
+    def cleanup(self, remove_old_logs_folder=False) -> None:
         """Cleanup the log and batch files.
 
         If the `RunManager` is not running, the ``run_script.py`` file
@@ -961,7 +936,15 @@ class RunManager(_BaseManager):
 
         _delete_old_ipython_profiles(self.scheduler)
 
-        cleanup(self.job_names, self.scheduler, True, self.move_old_logs_to)
+        cleanup(
+            job_names=self.job_names,
+            scheduler=self.scheduler,
+            with_progress_bar=True,
+            move_to=self.move_old_logs_to,
+        )
+        if remove_old_logs_folder:
+            with suppress(FileNotFoundError):
+                shutil.rmtree(self.move_old_logs_to)
 
     def parse_log_files(self, only_last: bool = True) -> pd.DataFrame:
         """Parse the log-files and convert it to a `~pandas.core.frame.DataFrame`.
@@ -997,7 +980,7 @@ class RunManager(_BaseManager):
 
     def load_learners(self) -> None:
         """Load the learners in parallel using `adaptive_scheduler.utils.load_parallel`."""
-        load_parallel(self.learners_module.learners, self.learners_module.fnames)
+        load_parallel(self.learners, self.fnames)
 
     def elapsed_time(self) -> float:
         """Total time elapsed since the `RunManager` was started."""
@@ -1034,161 +1017,11 @@ class RunManager(_BaseManager):
             self.end_time = time.time()
         return status
 
-    def info(self) -> None:
-        """Display information about the `RunManager`.
-
-        Returns an interactive ipywidget that can be
-        visualized in a Jupyter notebook.
-        """
-        from ipywidgets import Layout, Button, VBox, HTML
-        from IPython.display import display
-
-        status = HTML(value=self._info_html())
-
-        layout = Layout(width="200px")
-        buttons = [
-            Button(
-                description="update info",
-                layout=layout,
-                button_color="lightgreen",
-                icon="refresh",
-            ),
-            Button(
-                description="cancel jobs",
-                layout=layout,
-                button_style="danger",
-                icon="stop",
-            ),
-            Button(
-                description="cleanup log and batch files",
-                layout=layout,
-                button_style="danger",
-                icon="remove",
-            ),
-            Button(
-                description="load learners",
-                layout=layout,
-                button_style="info",
-                icon="download",
-            ),
-            Button(
-                description="show logs", layout=layout, button_style="info", icon="book"
-            ),
-        ]
-        buttons = {b.description: b for b in buttons}
-
-        box = VBox([])
-
-        log_widget = None
-
-        def update(_):
-            status.value = self._info_html()
-
-        def cancel(_):
-            self.cancel()
-            update(_)
-
-        def cleanup(_):
-            self.cleanup()
-            update(_)
-
-        def load_learners(_):
-            self.load_learners()
-
-        def toggle_logs(_):
-            nonlocal log_widget
-
-            if log_widget is None:
-                log_widget = log_explorer(self)
-
-            b = buttons["show logs"]
-            if b.description == "show logs":
-                b.description = "hide logs"
-                box.children = (*box.children, log_widget)
-            else:
-                b.description = "show logs"
-                box.children = box.children[:-1]
-
-        buttons["cancel jobs"].on_click(cancel)
-        buttons["cleanup log and batch files"].on_click(cleanup)
-        buttons["update info"].on_click(update)
-        buttons["show logs"].on_click(toggle_logs)
-        buttons["load learners"].on_click(load_learners)
-        box.children = (status, *tuple(buttons.values()))
-        display(box)
-
-    def _info_html(self) -> str:
-        queue = self.scheduler.queue(me_only=True)
-        self.database_manager.update(queue)
-        jobs = [job for job in queue.values() if job["job_name"] in self.job_names]
-        n_running = sum(job["state"] in ("RUNNING", "R") for job in jobs)
-        n_pending = sum(job["state"] in ("PENDING", "Q", "CONFIGURING") for job in jobs)
-        n_done = sum(job["is_done"] for job in self.database_manager.as_dicts())
-
-        status = self.status()
-        color = {
-            "cancelled": "orange",
-            "not yet started": "orange",
-            "running": "blue",
-            "failed": "red",
-            "finished": "green",
-        }[status]
-
-        def _table_row(i, key, value):
-            """Style the rows of a table. Based on the default Jupyterlab table style."""
-            style = "text-align: right; padding: 0.5em 0.5em; line-height: 1.0;"
-            if i % 2 == 1:
-                style += " background: var(--md-grey-100);"
-            return f'<tr><th style="{style}">{key}</th><th style="{style}">{value}</th></tr>'
-
-        info = [
-            ("status", f'<font color="{color}">{status}</font>'),
-            ("# running jobs", f'<font color="blue">{n_running}</font>'),
-            ("# pending jobs", f'<font color="orange">{n_pending}</font>'),
-            ("# finished jobs", f'<font color="green">{n_done}</font>'),
-            ("elapsed time", datetime.timedelta(seconds=self.elapsed_time())),
-        ]
-
-        with suppress(Exception):
-            df = self.parse_log_files()
-            t_last = (pd.Timestamp.now() - df.timestamp.max()).seconds
-
-            overhead = df.mem_usage.mean()
-            red_level = max(0, min(int(255 * overhead / 100), 255))
-            overhead_color = "#{:02x}{:02x}{:02x}".format(red_level, 255 - red_level, 0)
-            overhead_html_value = (
-                f'<font color="{overhead_color}">{overhead:.2f}%</font>'
-            )
-
-            cpu = df.cpu_usage.mean()
-            red_level = max(0, min(int(255 * cpu / 100), 255))
-            cpu_color = "#{:02x}{:02x}{:02x}".format(red_level, red_level, 0)
-            cpu_html_value = f'<font color="{cpu_color}">{cpu:.2f}%</font>'
-
-            from_logs = [
-                ("# of points", df.npoints.sum()),
-                ("mean CPU usage", cpu_html_value),
-                ("mean memory usage", f"{df.mem_usage.mean().round(1)} %"),
-                ("mean overhead", overhead_html_value,),
-                ("last log-entry", f"{t_last}s ago"),
-            ]
-            for key in ["npoints/s", "latest_loss", "nlearners"]:
-                with suppress(Exception):
-                    from_logs.append((f"mean {key}", f"{df[key].mean().round(1)}"))
-            msg = "this is extracted from the log files, so it might not be up-to-date"
-            abbr = '<abbr title="{}">{}</abbr>'  # creates a tooltip
-            info.extend([(abbr.format(msg, k), v) for k, v in from_logs])
-
-        table = "\n".join(_table_row(i, k, v) for i, (k, v) in enumerate(info))
-
-        return f"""
-            <table>
-            {table}
-            </table>
-        """
-
     def _repr_html_(self) -> None:
-        return self.info()
+        return info(self)
+
+    def info(self):
+        return info(self)
 
 
 def periodically_clean_ipython_profiles(scheduler, interval: int = 600):
