@@ -12,10 +12,13 @@ import time
 import warnings
 from concurrent.futures import ThreadPoolExecutor
 from contextlib import suppress
+from inspect import signature
+from multiprocessing import Manager
 from pathlib import Path
 from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
 
 import adaptive
+import cloudpickle
 import numpy as np
 import toolz
 from adaptive.notebook_integration import in_ipynb
@@ -23,6 +26,7 @@ from ipyparallel import Client
 from tqdm import tqdm, tqdm_notebook
 
 MAX_LINE_LENGTH = 100
+_NONE_RETURN_STR = "__ReturnsNone__"
 
 
 class _RequireAttrsABCMeta(abc.ABCMeta):
@@ -109,7 +113,7 @@ def split_in_balancing_learners(
 
 def split_sequence_learner(
     big_learner, n_learners: int, folder: Union[str, Path] = "",
-) -> Tuple[List[adaptive.SequenceLearner], List[Path]]:
+) -> Tuple[List[adaptive.SequenceLearner], List[str]]:
     r"""Split a sinlge `~adaptive.SequenceLearner` into
     mutiple `adaptive.SequenceLearner`\s (with the data loaded) and fnames.
 
@@ -129,7 +133,7 @@ def split_sequence_learner(
     new_learners : List[adaptive.SequenceLearner]
         List of `~adaptive.SequenceLearner`\s.
     new_fnames : List[Path]
-        List of `pathlib.Path`\s based on a hash of the sequence.
+        List of str based on a hash of the sequence.
     """
     new_learners, new_fnames = split_sequence_in_sequence_learners(
         function=big_learner._original_function,
@@ -155,7 +159,7 @@ def split_sequence_in_sequence_learners(
     sequence: Sequence[Any],
     n_learners: int,
     folder: Union[str, Path] = "",
-) -> Tuple[List[adaptive.SequenceLearner], List[Path]]:
+) -> Tuple[List[adaptive.SequenceLearner], List[str]]:
     r"""Split a sequenceinto `adaptive.SequenceLearner`\s and fnames.
 
     Parameters
@@ -174,7 +178,7 @@ def split_sequence_in_sequence_learners(
     new_learners : List[adaptive.SequenceLearner]
         List of `~adaptive.SequenceLearner`\s.
     new_fnames : List[Path]
-        List of `pathlib.Path`\s based on a hash of the sequence.
+        List of str based on a hash of the sequence.
     """
     folder = Path(folder)
     new_learners = []
@@ -184,7 +188,7 @@ def split_sequence_in_sequence_learners(
         new_learners.append(learner)
         hsh = hash_anything((sequence_part[0], len(sequence_part)))
         fname = folder / f"{hsh}.pickle"
-        new_fnames.append(fname)
+        new_fnames.append(str(fname))
     return new_learners, new_fnames
 
 
@@ -226,12 +230,35 @@ def combine_sequence_learners(
     return big_learner
 
 
+def copy_from_sequence_learner(
+    learner_from: adaptive.SequenceLearner, learner_to: adaptive.SequenceLearner
+) -> None:
+    """Convinience function to copy the data from a `~adaptive.SequenceLearner`
+    into a different `~adaptive.SequenceLearner`.
+
+    Parameters
+    ----------
+    learner_from : adaptive.SequenceLearner
+        Learner to take the data from.
+    learner_to : adaptive.SequenceLearner
+        Learner to tell the data to.
+    """
+    mapping = {
+        hash_anything(learner_from.sequence[i]): v for i, v in learner_from.data.items()
+    }
+    for i, key in enumerate(learner_to.sequence):
+        hsh = hash_anything(key)
+        if hsh in mapping:
+            v = mapping[hsh]
+            learner_to.tell((i, key), v)
+
+
 def _get_npoints(learner: adaptive.BaseLearner) -> Optional[int]:
     with suppress(AttributeError):
         return learner.npoints
     with suppress(AttributeError):
         # If the Learner is a BalancingLearner
-        return sum(l.npoints for l in learner.learners)
+        return sum(learner.npoints for learner in learner.learners)
 
 
 def _progress(
@@ -261,7 +288,7 @@ def combo2fname(
     folder: Optional[Union[str, Path]] = None,
     ext: Optional[str] = ".pickle",
     sig_figs: int = 8,
-) -> Path:
+) -> str:
     """Converts a dict into a human readable filename.
 
     Improved version of `combo_to_fname`."""
@@ -269,7 +296,7 @@ def combo2fname(
     fname = Path("__".join(name_parts) + ext)
     if folder is None:
         return fname
-    return folder / fname
+    return str(folder / fname)
 
 
 def add_constant_to_fname(
@@ -292,7 +319,14 @@ def add_constant_to_fname(
 
 def maybe_round(x: Any, sig_figs: int) -> Any:
     rnd = functools.partial(round_sigfigs, sig_figs=sig_figs)
-    if np.isnan(x) or np.isinf(x):
+
+    def try_is_nan_inf(x):
+        try:
+            return np.isnan(x) or np.isinf(x)
+        except Exception:
+            return False
+
+    if try_is_nan_inf(x):
         return x
     elif isinstance(x, (np.float, float)):
         return rnd(x)
@@ -342,7 +376,9 @@ def _remove_or_move_files(
                 os.remove(fname)
             else:
                 os.makedirs(move_to, exist_ok=True)
-                shutil.move(fname, Path(move_to) / fname)  # overwrites old files
+                src = Path(fname).resolve()
+                dst = (Path(move_to) / src.name).resolve()
+                shutil.move(src, dst)  # overwrites old files
         except Exception:
             n_failed += 1
 
@@ -490,3 +526,104 @@ def log_exception(log, msg, exception):
         raise exception
     except Exception:
         log.exception(msg, exc_info=True)
+
+
+def maybe_lst(fname: Union[List[str], str]):
+    if isinstance(fname, tuple):
+        # TinyDB converts tuples to lists
+        fname = list(fname)
+    return fname
+
+
+def _serialize(msg):
+    return [cloudpickle.dumps(msg)]
+
+
+def _deserialize(frames):
+    return cloudpickle.loads(frames[0])
+
+
+class LRUCachedCallable(Callable[..., Any]):
+    """Wraps a function to become cached.
+
+    Parameters
+    ----------
+    function : Callable[..., Any]
+    max_size : int, optional
+        Cache size of the LRU cache, by default 128.
+    """
+
+    def __init__(
+        self, function: Callable[..., Any], max_size: int = 128,
+    ):
+        self.max_size = max_size
+        self.function = function
+        self._signature = signature(self.function)
+        if max_size == 0:
+            return
+        manager = Manager()
+        self._cache_dict = manager.dict()
+        self._cache_queue = manager.list()
+        self._cache_lock = manager.Lock()
+
+    def _get_from_cache(self, key: str) -> Optional[Any]:
+        """Get a value from the cache by key."""
+        if self.max_size == 0:
+            value = None
+        with self._cache_lock:
+            value = self._cache_dict.get(key)
+            if value is not None:  # Move key to back of queue
+                self._cache_queue.remove(key)
+                self._cache_queue.append(key)
+        if value is not None:
+            found = True
+            value = value if value != _NONE_RETURN_STR else None
+        else:
+            found = False
+        return found, value
+
+    def _insert_into_cache(self, key: str, value: Any):
+        """Insert a key value pair into the cache."""
+        value = value if value is not None else _NONE_RETURN_STR
+        with self._cache_lock:
+            cache_size = len(self._cache_queue)
+            self._cache_dict[key] = value
+            if cache_size < self.max_size:
+                self._cache_queue.append(key)
+            else:
+                key_to_evict = self._cache_queue.pop(0)
+                self._cache_dict.pop(key_to_evict)
+                self._cache_queue.append(key)
+            return self._cache_queue
+
+    @property
+    def cache_dict(self):
+        """Returns a copy of the cache."""
+        return dict(self._cache_dict.items())
+
+    def __call__(self, *args, **kwargs) -> Any:
+        bound_args = self._signature.bind(*args, **kwargs)
+        bound_args.apply_defaults()
+        if self.max_size == 0:
+            return self.function(*args, **kwargs)
+        key = str(bound_args.arguments)
+        found, value = self._get_from_cache(key)
+        if found:
+            return value
+        ret = self.function(*args, **kwargs)
+        self._insert_into_cache(key, ret)
+        return ret
+
+
+def shared_memory_cache(cache_size: int = 128):
+    """Create a cache similar to `functools.lru_cache.
+
+    This will actually cache the return values of the function, whereas
+    `functools.lru_cache` will pickle the decorated function each time
+    with an empty cache.
+    """
+
+    def cache_decorator(function):
+        return functools.wraps(function)(LRUCachedCallable(function, cache_size))
+
+    return cache_decorator
